@@ -1,8 +1,6 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-"""
-Implements rotations, including spherical rotations as defined in WCS Paper II
-[1]_
+"""Implements rotations, including spherical rotations as defined in WCS Paper II [1]_.
 
 `RotateNative2Celestial` and `RotateCelestial2Native` follow the convention in
 WCS Paper II to rotate to/from a native sphere and the celestial sphere.
@@ -508,7 +506,6 @@ class Rotation2D(Model):
     @property
     def inverse(self):
         """Inverse rotation."""
-
         return self.__class__(angle=-self.angle)
 
     @classmethod
@@ -525,7 +522,6 @@ class Rotation2D(Model):
             If float, assumed in degrees.
 
         """
-
         if x.shape != y.shape:
             raise ValueError("Expected input arrays to have the same shape")
 
@@ -557,6 +553,8 @@ class Rotation2D(Model):
 
     @staticmethod
     def _compute_matrix(angle):
+        if not np.isscalar(angle):
+            angle = angle[0]
         return np.array(
             [[math.cos(angle), -math.sin(angle)], [math.sin(angle), math.cos(angle)]],
             dtype=np.float64,
