@@ -5,7 +5,7 @@
 import numpy as np
 
 from astropy import log
-from astropy.units import Unit, UnitConversionError, UnitsError
+from astropy.units import Unit, UnitConversionError, UnitsError  # noqa: F401
 
 from .flag_collection import FlagCollection
 from .mixins.ndarithmetic import NDArithmeticMixin
@@ -170,7 +170,9 @@ class NDDataArray(NDArithmeticMixin, NDSlicingMixin, NDIOMixin, NDData):
         if (value is not None) and (value is not np.ma.nomask):
             mask = np.array(value, dtype=np.bool_, copy=False)
             if mask.shape != self.data.shape:
-                raise ValueError("dimensions of mask do not match data")
+                raise ValueError(
+                    f"dimensions of mask {mask.shape} and data {self.data.shape} do not match"
+                )
             else:
                 self._mask = mask
         else:
@@ -201,7 +203,7 @@ class NDDataArray(NDArithmeticMixin, NDSlicingMixin, NDIOMixin, NDData):
     @property
     def ndim(self):
         """
-        integer dimensions of this object's data
+        integer dimensions of this object's data.
         """
         return self.data.ndim
 
