@@ -14,7 +14,7 @@ from numpy.testing import assert_allclose, assert_array_almost_equal, assert_arr
 from astropy import units as u
 from astropy.units.quantity import _UNIT_NOT_INITIALISED
 from astropy.utils import isiterable, minversion
-from astropy.utils.exceptions import AstropyDeprecationWarning, AstropyWarning
+from astropy.utils.exceptions import AstropyWarning
 
 """ The Quantity class will represent a number + unit + uncertainty """
 
@@ -397,6 +397,16 @@ class TestQuantityOperations:
         assert new_quantity.value == 171.3
         assert new_quantity.unit == u.meter
 
+        # Multiple with a unit.
+        new_quantity = self.q1 * u.s
+        assert new_quantity.value == 11.42
+        assert new_quantity.unit == u.Unit("m s")
+
+        # Reverse multiple with a unit.
+        new_quantity = u.s * self.q1
+        assert new_quantity.value == 11.42
+        assert new_quantity.unit == u.Unit("m s")
+
     def test_division(self):
         # Take units from left object, q1
         new_quantity = self.q1 / self.q2
@@ -424,6 +434,16 @@ class TestQuantityOperations:
         assert new_quantity.value == 1.0
         assert new_quantity.unit == u.Unit("1/m")
 
+        # Divide by a unit.
+        new_quantity = self.q1 / u.s
+        assert new_quantity.value == 11.42
+        assert new_quantity.unit == u.Unit("m/s")
+
+        # Divide into a unit.
+        new_quantity = u.s / self.q1
+        assert new_quantity.value == 1 / 11.42
+        assert new_quantity.unit == u.Unit("s/m")
+
     def test_commutativity(self):
         """Regression test for issue #587."""
 
@@ -505,11 +525,10 @@ class TestQuantityOperations:
 
     def test_non_number_type(self):
         q1 = u.Quantity(11.412, unit=u.meter)
-        with pytest.raises(TypeError) as exc:
+        with pytest.raises(
+            TypeError, match=r"Unsupported operand type\(s\) for ufunc .*"
+        ):
             q1 + {"a": 1}
-        assert exc.value.args[0].startswith(
-            "Unsupported operand type(s) for ufunc add:"
-        )
 
         with pytest.raises(TypeError):
             q1 + u.meter
@@ -588,15 +607,8 @@ class TestQuantityOperations:
         assert not 1.0 * u.cm == 1.0
         assert 1.0 * u.cm != 1.0
 
-        # comparison with zero should raise a deprecation warning
         for quantity in (1.0 * u.cm, 1.0 * u.dimensionless_unscaled):
-            with pytest.warns(
-                AstropyDeprecationWarning,
-                match=(
-                    "The truth value of a Quantity is ambiguous. "
-                    "In the future this will raise a ValueError."
-                ),
-            ):
+            with pytest.raises(ValueError, match="ambiguous"):
                 bool(quantity)
 
     def test_numeric_converters(self):
