@@ -6,21 +6,21 @@
 # which are then converted by setup.py into docstrings.h, which is
 # included by pywcs.c
 
-__all__ = ['TWO_OR_MORE_ARGS', 'RETURNS', 'ORIGIN', 'RA_DEC_ORDER']
+__all__ = ["TWO_OR_MORE_ARGS", "RETURNS", "ORIGIN", "RA_DEC_ORDER"]
 
 
 def _fix(content, indent=0):
-    lines = content.split('\n')
-    indent = '\n' + ' ' * indent
+    lines = content.split("\n")
+    indent = "\n" + " " * indent
     return indent.join(lines)
 
 
 def TWO_OR_MORE_ARGS(naxis, indent=0):
     return _fix(
-"""args : flexible
+        f"""*args
     There are two accepted forms for the positional arguments:
 
-        - 2 arguments: An *N* x *{}* array of coordinates, and an
+        - 2 arguments: An *N* x *{naxis}* array of coordinates, and an
           *origin*.
 
         - more than 2 arguments: An array for each axis, followed by
@@ -30,35 +30,44 @@ def TWO_OR_MORE_ARGS(naxis, indent=0):
     Here, *origin* is the coordinate in the upper left corner of the
     image.  In FITS and Fortran standards, this is 1.  In Numpy and C
     standards this is 0.
-""".format(naxis), indent)
+""",
+        indent,
+    )
 
 
 def RETURNS(out_type, indent=0):
-    return _fix("""result : array
-    Returns the {}.  If the input was a single array and
+    return _fix(
+        f"""result : array
+    Returns the {out_type}.  If the input was a single array and
     origin, a single array is returned, otherwise a tuple of arrays is
-    returned.""".format(out_type), indent)
+    returned.""",
+        indent,
+    )
 
 
 def ORIGIN(indent=0):
     return _fix(
-"""
+        """
 origin : int
     Specifies the origin of pixel values.  The Fortran and FITS
     standards use an origin of 1.  Numpy and C use array indexing with
     origin at 0.
-""", indent)
+""",
+        indent,
+    )
 
 
 def RA_DEC_ORDER(indent=0):
     return _fix(
-"""
+        """
 ra_dec_order : bool, optional
     When `True` will ensure that world coordinates are always given
     and returned in as (*ra*, *dec*) pairs, regardless of the order of
     the axes specified by the in the ``CTYPE`` keywords.  Default is
     `False`.
-""", indent)
+""",
+        indent,
+    )
 
 
 a = """
@@ -76,7 +85,7 @@ a_order = """
 ``int`` (read-only) Order of the polynomial (``A_ORDER``).
 """
 
-all_pix2world = """
+all_pix2world = f"""
 all_pix2world(pixcrd, origin) -> ``double array[ncoord][nelem]``
 
 Transforms pixel coordinates to world coordinates.
@@ -95,15 +104,15 @@ The first three (the distortion corrections) are done in parallel.
 
 Parameters
 ----------
-pixcrd : double array[ncoord][nelem]
-    Array of pixel coordinates.
+pixcrd : ndarray
+    Array of pixel coordinates as ``double array[ncoord][nelem]``.
 
-{}
+{ORIGIN()}
 
 Returns
 -------
-world : double array[ncoord][nelem]
-    Returns an array of world coordinates.
+world : ndarray
+    Returns an array of world coordinates as ``double array[ncoord][nelem]``.
 
 Raises
 ------
@@ -130,7 +139,7 @@ InvalidTransformError
 
 InvalidTransformError
     Ill-conditioned coordinate transformation parameters.
-""".format(ORIGIN())
+"""
 
 alt = """
 ``str`` Character code for alternate coordinate descriptions.
@@ -153,6 +162,26 @@ ap_order = """
 ``int`` (read-only) Order of the polynomial (``AP_ORDER``).
 """
 
+cel = """
+`~astropy.wcs.Celprm` Information required to transform celestial coordinates.
+"""
+
+Celprm = """
+Class that contains information required to transform celestial coordinates.
+It consists of certain members that must be set by the user (given) and others
+that are set by the WCSLIB routines (returned).
+Some of the latter are supplied for informational purposes and others are for
+internal use only.
+"""
+
+Prjprm = """
+Class that contains information needed to project or deproject native spherical coordinates.
+It consists of certain members that must be set by the user (given) and others
+that are set by the WCSLIB routines (returned).
+Some of the latter are supplied for informational purposes and others are for
+internal use only.
+"""
+
 aux = """
 `~astropy.wcs.Auxprm` Auxiliary coordinate system information of a specialist nature.
 """
@@ -320,6 +349,317 @@ If `True`, an offset will be applied to ``(x, y)`` to force ``(x, y) =
 (0, 0)`` at the fiducial point, (phi_0, theta_0).  Default is `False`.
 """
 
+celprm_phi0 = r"""
+`float`, `None`. The native longitude, :math:`\phi_0`, in degrees of the
+fiducial point, i.e., the point whose celestial coordinates are given in
+''Celprm.ref[0:1]''. If `None` or ``nan``, the initialization routine,
+``celset()``, will set this to a projection-specific default.
+"""
+
+celprm_theta0 = r"""
+`float`, `None`. The native latitude, :math:`\theta_0`, in degrees of the
+fiducial point, i.e. the point whose celestial coordinates are given in
+``Celprm:ref[0:1]``. If `None` or ``nan``, the initialization routine,
+``celset()``, will set this to a projection-specific default.
+"""
+
+celprm_ref = """
+``numpy.ndarray`` with 4 elements.
+(Given) The first pair of values should be set to the celestial longitude and
+latitude of the fiducial point in degrees - typically right ascension and
+declination. These are given by the ``CRVALia`` keywords in ``FITS``.
+
+(Given and returned) The second pair of values are the native longitude,
+``phi_p`` (in degrees), and latitude, ``theta_p`` (in degrees), of the
+celestial pole (the latter is the same as the celestial latitude of the
+native pole, ``delta_p``) and these are given by the ``FITS`` keywords
+``LONPOLEa`` and ``LATPOLEa`` (or by ``PVi_2a`` and ``PVi_3a`` attached
+to the longitude axis which take precedence if defined).
+
+``LONPOLEa`` defaults to ``phi0`` if the celestial latitude of the fiducial
+point of the projection is greater than or equal to the native latitude,
+otherwise ``phi0 + 180`` (degrees). (This is the condition for the celestial
+latitude to increase in the same direction as the native latitude at the
+fiducial point.) ``ref[2]`` may be set to `None` or ``numpy.nan``
+or 999.0 to indicate that the correct default should be substituted.
+
+``theta_p``, the native latitude of the celestial pole (or equally the
+celestial latitude of the native pole, ``delta_p``) is often determined
+uniquely by ``CRVALia`` and ``LONPOLEa`` in which case ``LATPOLEa`` is ignored.
+However, in some circumstances there are two valid solutions for ``theta_p``
+and ``LATPOLEa`` is used to choose between them. ``LATPOLEa`` is set in
+``ref[3]`` and the solution closest to this value is used to reset ``ref[3]``.
+It is therefore legitimate, for example, to set ``ref[3]`` to ``+90.0``
+to choose the more northerly solution - the default if the ``LATPOLEa`` keyword
+is omitted from the ``FITS`` header. For the special case where the fiducial
+point of the projection is at native latitude zero, its celestial latitude
+is zero, and ``LONPOLEa`` = ``+/- 90.0`` then the celestial latitude of the
+native pole is not determined by the first three reference values and
+``LATPOLEa`` specifies it completely.
+
+The returned value, celprm.latpreq, specifies how ``LATPOLEa``
+was actually used."""
+
+celprm_euler = """
+*Read-only* ``numpy.ndarray`` with 5 elements. Euler angles and associated
+intermediaries derived from the coordinate reference values. The first three
+values are the ``Z-``, ``X-``, and ``Z``-Euler angles in degrees, and the
+remaining two are the cosine and sine of the ``X``-Euler angle.
+"""
+
+celprm_latpreq = """
+``int``, *read-only*. For informational purposes, this indicates how the
+``LATPOLEa`` keyword was used:
+
+- 0: Not required, ``theta_p == delta_p`` was determined uniquely by the
+    ``CRVALia`` and ``LONPOLEa`` keywords.
+- 1: Required to select between two valid solutions of ``theta_p``.
+- 2: ``theta_p`` was specified solely by ``LATPOLEa``.
+"""
+
+celprm_isolat = """
+``bool``, *read-only*. True if the spherical rotation preserves the magnitude
+of the latitude, which occurs if the axes of the native and celestial
+coordinates are coincident. It signals an opportunity to cache intermediate
+calculations common to all elements in a vector computation.
+"""
+
+celprm_prj = """
+*Read-only* Celestial transformation parameters. Some members of `Prjprm`
+are read-write, i.e., can be set by the user. For more details, see
+documentation for `Prjprm`.
+"""
+
+prjprm_r0 = r"""
+The radius of the generating sphere for the projection, a linear scaling
+parameter. If this is zero, it will be reset to its default value of
+:math:`180^\circ/\pi` (the value for FITS WCS).
+"""
+
+prjprm_code = """
+Three-letter projection code defined by the FITS standard.
+"""
+
+prjprm_pv = """
+Projection parameters. These correspond to the ``PVi_ma`` keywords in FITS,
+so ``pv[0]`` is ``PVi_0a``, ``pv[1]`` is ``PVi_1a``, etc., where ``i`` denotes
+the latitude-like axis. Many projections use ``pv[1]`` (``PVi_1a``),
+some also use ``pv[2]`` (``PVi_2a``) and ``SZP`` uses ``pv[3]`` (``PVi_3a``).
+``ZPN`` is currently the only projection that uses any of the others.
+
+When setting ``pv`` values using lists or ``numpy.ndarray``,
+elements set to `None` will be left unchanged while those set to ``numpy.nan``
+will be set to ``WCSLIB``'s ``UNDEFINED`` special value. For efficiency
+purposes, if supplied list or ``numpy.ndarray`` is shorter than the length of
+the ``pv`` member, then remaining values in ``pv`` will be left unchanged.
+
+.. note::
+    When retrieving ``pv``, a copy of the ``prjprm.pv`` array is returned.
+    Modifying this array values will not modify underlying ``WCSLIB``'s
+    ``prjprm.pv`` data.
+"""
+
+prjprm_pvi = """
+Set/Get projection parameters for specific index. These correspond to the
+``PVi_ma`` keywords in FITS, so ``pv[0]`` is ``PVi_0a``, ``pv[1]`` is
+``PVi_1a``, etc., where ``i`` denotes the latitude-like axis.
+Many projections use ``pv[1]`` (``PVi_1a``),
+some also use ``pv[2]`` (``PVi_2a``) and ``SZP`` uses ``pv[3]`` (``PVi_3a``).
+``ZPN`` is currently the only projection that uses any of the others.
+
+Setting a ``pvi`` value to `None` will reset the corresponding ``WCSLIB``'s
+``prjprm.pv`` element to the default value as set by ``WCSLIB``'s ``prjini()``.
+
+Setting a ``pvi`` value to ``numpy.nan`` will set the corresponding
+``WCSLIB``'s ``prjprm.pv`` element to ``WCSLIB``'s ``UNDEFINED`` special value.
+"""
+
+prjprm_phi0 = r"""
+The native longitude, :math:`\phi_0` (in degrees) of the reference point,
+i.e. the point ``(x,y) = (0,0)``. If undefined the initialization routine
+will set this to a projection-specific default.
+"""
+
+prjprm_theta0 = r"""
+the native latitude, :math:`\theta_0` (in degrees) of the reference point,
+i.e. the point ``(x,y) = (0,0)``. If undefined the initialization routine
+will set this to a projection-specific default.
+"""
+
+prjprm_bounds = """
+Controls bounds checking. If ``bounds&1`` then enable strict bounds checking
+for the spherical-to-Cartesian (``s2x``) transformation for the
+``AZP``, ``SZP``, ``TAN``, ``SIN``, ``ZPN``, and ``COP`` projections.
+If ``bounds&2`` then enable strict bounds checking for the
+Cartesian-to-spherical transformation (``x2s``) for the ``HPX`` and ``XPH``
+projections. If ``bounds&4`` then the Cartesian- to-spherical transformations
+(``x2s``) will invoke WCSLIB's ``prjbchk()`` to perform bounds checking on the
+computed native coordinates, with a tolerance set to suit each projection.
+bounds is set to 7 during initialization by default which enables all checks.
+Zero it to disable all checking.
+
+It is not necessary to reset the ``Prjprm`` struct (via ``Prjprm.set()``) when
+``bounds`` is changed.
+"""
+
+prjprm_name = """
+*Read-only.* Long name of the projection.
+"""
+
+prjprm_category = """
+*Read-only.* Projection category matching the value of the relevant ``wcs``
+module constants:
+
+PRJ_ZENITHAL,
+PRJ_CYLINDRICAL,
+PRJ_PSEUDOCYLINDRICAL,
+PRJ_CONVENTIONAL,
+PRJ_CONIC,
+PRJ_POLYCONIC,
+PRJ_QUADCUBE, and
+PRJ_HEALPIX.
+"""
+
+prjprm_w = """
+*Read-only.* Intermediate floating-point values derived from the projection
+parameters, cached here to save recomputation.
+
+.. note::
+    When retrieving ``w``, a copy of the ``prjprm.w`` array is returned.
+    Modifying this array values will not modify underlying ``WCSLIB``'s
+    ``prjprm.w`` data.
+
+"""
+
+prjprm_pvrange = """
+*Read-only.* Range of projection parameter indices: 100 times the first allowed
+index plus the number of parameters, e.g. ``TAN`` is 0 (no parameters),
+``SZP`` is 103 (1 to 3), and ``ZPN`` is 30 (0 to 29).
+"""
+
+prjprm_simplezen = """
+*Read-only.* True if the projection is a radially-symmetric zenithal projection.
+"""
+
+prjprm_equiareal = """
+*Read-only.* True if the projection is equal area.
+"""
+
+prjprm_conformal = """
+*Read-only.* True if the projection is conformal.
+"""
+
+prjprm_global_projection = """
+*Read-only.* True if the projection can represent the whole sphere in a finite,
+non-overlapped mapping.
+"""
+
+prjprm_divergent = """
+*Read-only.* True if the projection diverges in latitude.
+"""
+
+prjprm_x0 = r"""
+*Read-only.* The offset in ``x`` used to force :math:`(x,y) = (0,0)` at
+:math:`(\phi_0, \theta_0)`.
+"""
+
+prjprm_y0 = r"""
+*Read-only.* The offset in ``y`` used to force :math:`(x,y) = (0,0)` at
+:math:`(\phi_0, \theta_0)`.
+"""
+
+prjprm_m = """
+*Read-only.* Intermediate integer value (used only for the ``ZPN`` and ``HPX`` projections).
+"""
+
+prjprm_n = """
+*Read-only.* Intermediate integer value (used only for the ``ZPN`` and ``HPX`` projections).
+"""
+
+prjprm_set = """
+This method sets up a ``Prjprm`` object according to information supplied
+within it.
+
+Note that this routine need not be called directly; it will be invoked by
+`prjx2s` and `prjs2x` if ``Prjprm.flag`` is anything other than a predefined
+magic value.
+
+The one important property of ``set()`` is that the projection code must be
+defined in the ``Prjprm`` in order for ``set()`` to identify the required
+projection.
+
+Raises
+------
+MemoryError
+    Null ``prjprm`` pointer passed to WCSLIB routines.
+
+InvalidPrjParametersError
+    Invalid projection parameters.
+
+InvalidCoordinateError
+    One or more of the ``(x,y)`` or ``(lon,lat)`` coordinates were invalid.
+"""
+
+prjprm_prjx2s = r"""
+Deproject Cartesian ``(x,y)`` coordinates in the plane of projection to native
+spherical coordinates :math:`(\phi,\theta)`.
+
+The projection is that specified by ``Prjprm.code``.
+
+Parameters
+----------
+x, y : numpy.ndarray
+    Arrays corresponding to the first (``x``) and second (``y``) projected
+    coordinates.
+
+Returns
+-------
+phi, theta : tuple of numpy.ndarray
+    Longitude and latitude :math:`(\phi,\theta)` of the projected point in
+    native spherical coordinates (in degrees). Values corresponding to
+    invalid ``(x,y)`` coordinates are set to ``numpy.nan``.
+
+Raises
+------
+MemoryError
+    Null ``prjprm`` pointer passed to WCSLIB routines.
+
+InvalidPrjParametersError
+    Invalid projection parameters.
+
+"""
+
+prjprm_prjs2x = r"""
+Project native spherical coordinates :math:`(\phi,\theta)` to Cartesian
+``(x,y)`` coordinates in the plane of projection.
+
+The projection is that specified by ``Prjprm.code``.
+
+Parameters
+----------
+phi : numpy.ndarray
+    Array corresponding to the longitude :math:`\phi` of the projected point
+    in native spherical coordinates (in degrees).
+theta : numpy.ndarray
+    Array corresponding to the longitude :math:`\theta` of the projected point
+    in native spherical coordinates (in degrees). Values corresponding to
+    invalid :math:`(\phi, \theta)` coordinates are set to ``numpy.nan``.
+
+Returns
+-------
+x, y : tuple of numpy.ndarray
+    Projected coordinates.
+
+Raises
+------
+MemoryError
+    Null ``prjprm`` pointer passed to WCSLIB routines.
+
+InvalidPrjParametersError
+    Invalid projection parameters.
+
+"""
+
 celfix = """
 Translates AIPS-convention celestial projection types, ``-NCP`` and
 ``-GLS``.
@@ -365,12 +705,13 @@ cmp : int, optional
     A bit field controlling the strictness of the comparison.  When 0,
     (the default), all fields must be identical.
 
-    The following constants may be or'ed together to loosen the
-    comparison.
+    The following constants, defined in the `astropy.wcs` module,
+    may be or'ed together to loosen the comparison.
 
     - ``WCSCOMPARE_ANCILLARY``: Ignores ancillary keywords that don't
-      change the WCS transformation, such as ``DATE-OBS`` or
-      ``EQUINOX``.
+      change the WCS transformation, such as ``XPOSURE`` or
+      ``EQUINOX``. Note that this also ignores ``DATE-OBS``, which does
+      change the WCS transformation in some cases.
 
     - ``WCSCOMPARE_TILING``: Ignore integral differences in
       ``CRPIXja``.  This is the 'tiling' condition, where two WCSes
@@ -688,15 +1029,11 @@ An undefined value is represented by NaN.
 """
 
 extlev = """
-``int`` (read-only)
-
-``EXTLEV`` identifying the binary table extension.
+``int`` (read-only) ``EXTLEV`` identifying the binary table extension.
 """
 
 extnam = """
-``str`` (read-only)
-
-``EXTNAME`` identifying the binary table extension.
+``str`` (read-only) ``EXTNAME`` identifying the binary table extension.
 """
 
 extrema = """
@@ -715,9 +1052,7 @@ speed up table searches.
 """
 
 extver = """
-``int`` (read-only)
-
-``EXTVER`` identifying the binary table extension.
+``int`` (read-only) ``EXTVER`` identifying the binary table extension.
 """
 
 find_all_wcs = """
@@ -741,7 +1076,7 @@ relax : bool or int
       standard.
 
     - `int`: a bit field selecting specific extensions to accept.  See
-      :ref:`relaxread` for details.
+      :ref:`astropy:relaxread` for details.
 
 keysel : sequence of flags
     Used to restrict the keyword types considered:
@@ -757,7 +1092,7 @@ keysel : sequence of flags
 
 Returns
 -------
-wcs_list : list of `~astropy.wcs.Wcsprm` objects
+wcs_list : list of `~astropy.wcs.Wcsprm`
 """
 
 fix = """
@@ -793,7 +1128,7 @@ translate_units : str, optional
     Thus ``''`` doesn't do any unsafe translations, whereas ``'shd'``
     does all of them.
 
-naxis : int array[naxis], optional
+naxis : int array, optional
     Image axis lengths.  If this array is set to zero or ``None``,
     then `~astropy.wcs.Wcsprm.cylfix` will not be invoked.
 
@@ -825,7 +1160,7 @@ Returns the offset as defined in the distortion lookup table.
 
 Returns
 -------
-coordinate : coordinate pair
+coordinate : (2,) tuple
     The offset from the distortion table for pixel point (*x*, *y*).
 """
 
@@ -882,7 +1217,7 @@ Returns ``PVi_ma`` keywords for each *i* and *m* as list of tuples.
 
 Returns
 -------
-
+sequence of tuple
     Returned as a list of tuples of the form (*i*, *m*, *value*):
 
     - *i*: int.  Axis number, as in ``PVi_ma``, (i.e. 1-relative)
@@ -995,9 +1330,7 @@ hglt_obs = """
 """
 
 i = """
-``int`` (read-only)
-
-Image axis number.
+``int`` (read-only) Image axis number.
 """
 
 imgpix_matrix = """
@@ -1024,9 +1357,9 @@ the coordinate array and of each indexing vector.
 """
 
 kind = """
-``str`` (read-only)
+``str`` (read-only) ``wcstab`` array type.
 
-Character identifying the wcstab array type:
+Character identifying the ``wcstab`` array type:
 
     - ``'c'``: coordinate array,
     - ``'i'``: index vector.
@@ -1073,9 +1406,7 @@ M = """
 """
 
 m = """
-``int`` (read-only)
-
-Array axis number for index vectors.
+``int`` (read-only) ``wcstab`` axis number for index vectors.
 """
 
 map = """
@@ -1101,7 +1432,7 @@ coordinates with the relevant coordinate element with ``nelem`` set to
 1.  In this case ``map[0] = 0`` regardless of the value of *i*.
 """
 
-mix = """
+mix = f"""
 mix(mixpix, mixcel, vspan, vstep, viter, world, pixcrd, origin)
 
 Given either the celestial longitude or latitude plus an element of
@@ -1121,7 +1452,7 @@ mixcel : int
     celestial latitude is given in ``world[self.lat]``, longitude
     returned in ``world[self.lng]``.
 
-vspan : pair of floats
+vspan : (float, float)
     Solution interval for the celestial coordinate, in degrees.  The
     ordering of the two limits is irrelevant.  Longitude ranges may be
     specified with any convenient normalization, for example
@@ -1138,18 +1469,18 @@ viter : int
     the search recommenced.  *viter* controls how many times the step
     size is halved.  The allowed range is 5 - 10.
 
-world : double array[naxis]
-    World coordinate elements.  ``world[self.lng]`` and
+world : ndarray
+    World coordinate elements as ``double array[naxis]``.  ``world[self.lng]`` and
     ``world[self.lat]`` are the celestial longitude and latitude, in
     degrees.  Which is given and which returned depends on the value
     of *mixcel*.  All other elements are given.  The results will be
     written to this array in-place.
 
-pixcrd : double array[naxis].
-    Pixel coordinates.  The element indicated by *mixpix* is given and
+pixcrd : ndarray
+    Pixel coordinates as ``double array[naxis]``.  The element indicated by *mixpix* is given and
     the remaining elements will be written in-place.
 
-{}
+{ORIGIN()}
 
 Returns
 -------
@@ -1157,20 +1488,20 @@ result : dict
 
     Returns a dictionary with the following keys:
 
-    - *phi* (double array[naxis])
+    - *phi* (``double array[naxis]``)
 
-    - *theta* (double array[naxis])
+    - *theta* (``double array[naxis]``)
 
         - Longitude and latitude in the native coordinate system of
           the projection, in degrees.
 
-    - *imgcrd* (double array[naxis])
+    - *imgcrd* (``double array[naxis]``)
 
         - Image coordinate elements.  ``imgcrd[self.lng]`` and
           ``imgcrd[self.lat]`` are the projected *x*- and
           *y*-coordinates, in decimal degrees.
 
-    - *world* (double array[naxis])
+    - *world* (``double array[naxis]``)
 
         - Another reference to the *world* argument passed in.
 
@@ -1235,7 +1566,7 @@ Because of its generality, `~astropy.wcs.Wcsprm.mix` is very
 compute-intensive.  For compute-limited applications, more efficient
 special-case solvers could be written for simple projections, for
 example non-oblique cylindrical projections.
-""".format(ORIGIN())
+"""
 
 mjdavg = """
 ``double`` Modified Julian Date corresponding to ``DATE-AVG``.
@@ -1300,9 +1631,7 @@ product K_1 * K_2 * ... * K_M.
 """
 
 ndim = """
-``int`` (read-only)
-
-Expected dimensionality of the wcstab array.
+``int`` (read-only) Expected dimensionality of the ``wcstab`` array.
 """
 
 obsgeo = """
@@ -1322,7 +1651,7 @@ indices into the coordinate array such that Upsilon_m, as defined in
 Paper III, is equal to ``(p0[m] + 1) + delta[m]``.
 """
 
-p2s = """
+p2s = f"""
 p2s(pixcrd, origin)
 
 Converts pixel to world coordinates.
@@ -1330,42 +1659,44 @@ Converts pixel to world coordinates.
 Parameters
 ----------
 
-pixcrd : double array[ncoord][nelem]
-    Array of pixel coordinates.
+pixcrd : ndarray
+    Array of pixel coordinates as ``double array[ncoord][nelem]``.
 
-{}
+{ORIGIN()}
 
 Returns
 -------
 result : dict
     Returns a dictionary with the following keys:
 
-    - *imgcrd*: double array[ncoord][nelem]
+    - *imgcrd*: ndarray
 
-      - Array of intermediate world coordinates.  For celestial axes,
+      - Array of intermediate world coordinates as ``double array[ncoord][nelem]``.  For celestial axes,
         ``imgcrd[][self.lng]`` and ``imgcrd[][self.lat]`` are the
         projected *x*-, and *y*-coordinates, in pseudo degrees.  For
         spectral axes, ``imgcrd[][self.spec]`` is the intermediate
         spectral coordinate, in SI units.
 
-    - *phi*: double array[ncoord]
+    - *phi*: ndarray
 
-    - *theta*: double array[ncoord]
+      - Array as ``double array[ncoord]``.
+
+    - *theta*: ndarray
 
       - Longitude and latitude in the native coordinate system of the
-        projection, in degrees.
+        projection, in degrees, as ``double array[ncoord]``.
 
-    - *world*: double array[ncoord][nelem]
+    - *world*: ndarray
 
-      - Array of world coordinates.  For celestial axes,
+      - Array of world coordinates as ``double array[ncoord][nelem]``.  For celestial axes,
         ``world[][self.lng]`` and ``world[][self.lat]`` are the
         celestial longitude and latitude, in degrees.  For spectral
         axes, ``world[][self.spec]`` is the intermediate spectral
         coordinate, in SI units.
 
-    - *stat*: int array[ncoord]
+    - *stat*: ndarray
 
-      - Status return value for each coordinate. ``0`` for success,
+      - Status return value for each coordinate as ``int array[ncoord]``. ``0`` for success,
         ``1+`` for invalid pixel coordinate.
 
 Raises
@@ -1396,25 +1727,25 @@ See also
 --------
 astropy.wcs.Wcsprm.lat, astropy.wcs.Wcsprm.lng
     Definition of the latitude and longitude axes
-""".format(ORIGIN())
+"""
 
-p4_pix2foc = """
-p4_pix2foc(*pixcrd, origin*) -> double array[ncoord][nelem]
+p4_pix2foc = f"""
+p4_pix2foc(*pixcrd, origin*) -> ``double array[ncoord][nelem]``
 
 Convert pixel coordinates to focal plane coordinates using `distortion
 paper`_ lookup-table correction.
 
 Parameters
 ----------
-pixcrd : double array[ncoord][nelem].
-    Array of pixel coordinates.
+pixcrd : ndarray
+    Array of pixel coordinates as ``double array[ncoord][nelem]``.
 
-{}
+{ORIGIN()}
 
 Returns
 -------
-foccrd : double array[ncoord][nelem]
-    Returns an array of focal plane coordinates.
+foccrd : ndarray
+    Returns an array of focal plane coordinates as ``double array[ncoord][nelem]``.
 
 Raises
 ------
@@ -1423,7 +1754,7 @@ MemoryError
 
 ValueError
     Invalid coordinate transformation parameters.
-""".format(ORIGIN())
+"""
 
 pc = """
 ``double array[naxis][naxis]`` The ``PCi_ja`` (pixel coordinate)
@@ -1466,23 +1797,23 @@ See also
 astropy.wcs.Wcsprm.theta0
 """
 
-pix2foc = """
-pix2foc(*pixcrd, origin*) -> double array[ncoord][nelem]
+pix2foc = f"""
+pix2foc(*pixcrd, origin*) -> ``double array[ncoord][nelem]``
 
 Perform both `SIP`_ polynomial and `distortion paper`_ lookup-table
 correction in parallel.
 
 Parameters
 ----------
-pixcrd : double array[ncoord][nelem]
-    Array of pixel coordinates.
+pixcrd : ndarray
+    Array of pixel coordinates as ``double array[ncoord][nelem]``.
 
-{}
+{ORIGIN()}
 
 Returns
 -------
-foccrd : double array[ncoord][nelem]
-    Returns an array of focal plane coordinates.
+foccrd : ndarray
+    Returns an array of focal plane coordinates as ``double array[ncoord][nelem]``.
 
 Raises
 ------
@@ -1491,7 +1822,7 @@ MemoryError
 
 ValueError
     Invalid coordinate transformation parameters.
-""".format(ORIGIN())
+"""
 
 piximg_matrix = """
 ``double array[2][2]`` (read-only) Matrix containing the product of
@@ -1518,6 +1849,16 @@ removed in the future.
 To get a string of the contents, use `repr`.
 """
 
+print_contents_wtbarr = """
+print_contents()
+
+Print the contents of the `~astropy.wcs.Wtbarr` object to
+stdout. Probably only useful for debugging purposes, and may be
+removed in the future.
+
+To get a string of the contents, use `repr`.
+"""
+
 radesys = """
 ``string`` The equatorial or ecliptic coordinate system type,
 ``RADESYSa``.
@@ -1536,9 +1877,7 @@ An undefined value is represented by NaN.
 """
 
 row = """
-``int`` (read-only)
-
-Table row number.
+``int`` (read-only) Table row number.
 """
 
 rsun_ref = """
@@ -1546,31 +1885,31 @@ rsun_ref = """
 If undefined, this is set to `None`.
 """
 
-s2p = """
+s2p = f"""
 s2p(world, origin)
 
 Transforms world coordinates to pixel coordinates.
 
 Parameters
 ----------
-world : double array[ncoord][nelem]
-    Array of world coordinates, in decimal degrees.
+world : ndarray
+    Array of world coordinates, in decimal degrees, as ``double array[ncoord][nelem]``.
 
-{}
+{ORIGIN()}
 
 Returns
 -------
 result : dict
     Returns a dictionary with the following keys:
 
-    - *phi*: double array[ncoord]
+    - *phi*: ``double array[ncoord]``
 
-    - *theta*: double array[ncoord]
+    - *theta*: ``double array[ncoord]``
 
         - Longitude and latitude in the native coordinate system of
           the projection, in degrees.
 
-    - *imgcrd*: double array[ncoord][nelem]
+    - *imgcrd*: ``double array[ncoord][nelem]``
 
        - Array of intermediate world coordinates.  For celestial axes,
          ``imgcrd[][self.lng]`` and ``imgcrd[][self.lat]`` are the
@@ -1580,12 +1919,12 @@ result : dict
          spectral axes, ``imgcrd[][self.spec]`` is the intermediate
          spectral coordinate, in SI units.
 
-    - *pixcrd*: double array[ncoord][nelem]
+    - *pixcrd*: ``double array[ncoord][nelem]``
 
         - Array of pixel coordinates.  Pixel coordinates are
           zero-based.
 
-    - *stat*: int array[ncoord]
+    - *stat*: ``int array[ncoord]``
 
         - Status return value for each coordinate. ``0`` for success,
           ``1+`` for invalid pixel coordinate.
@@ -1614,7 +1953,7 @@ See also
 --------
 astropy.wcs.Wcsprm.lat, astropy.wcs.Wcsprm.lng
     Definition of the latitude and longitude axes
-""".format(ORIGIN())
+"""
 
 sense = """
 ``int array[M]`` +1 if monotonically increasing, -1 if decreasing.
@@ -1683,10 +2022,27 @@ Raises
 MemoryError
     Memory allocation failed.
 
-InvalidTabularParameters
+InvalidTabularParametersError
     Invalid tabular parameters.
 """
 
+set_celprm = """
+set()
+
+Sets up a ``celprm`` struct according to information supplied within it.
+
+Note that this routine need not be called directly; it will be invoked
+by functions that need it.
+
+Raises
+------
+MemoryError
+    Memory allocation failed.
+
+InvalidPrjParametersError
+    Invalid celestial parameters.
+"""
+
 set_ps = """
 set_ps(ps)
 
@@ -1694,7 +2050,7 @@ Sets ``PSi_ma`` keywords for each *i* and *m*.
 
 Parameters
 ----------
-ps : sequence of tuples
+ps : sequence of tuple
 
     The input must be a sequence of tuples of the form (*i*, *m*,
     *value*):
@@ -1717,7 +2073,7 @@ Sets ``PVi_ma`` keywords for each *i* and *m*.
 
 Parameters
 ----------
-pv : list of tuples
+pv : list of tuple
 
     The input must be a sequence of tuples of the form (*i*, *m*,
     *value*):
@@ -1746,24 +2102,24 @@ using the `SIP`_ convention in both directions.
 
 Parameters
 ----------
-a : double array[m+1][m+1]
-    The ``A_i_j`` polynomial for pixel to focal plane transformation.
+a : ndarray
+    The ``A_i_j`` polynomial for pixel to focal plane transformation as ``double array[m+1][m+1]``.
     Its size must be (*m* + 1, *m* + 1) where *m* = ``A_ORDER``.
 
-b : double array[m+1][m+1]
-    The ``B_i_j`` polynomial for pixel to focal plane transformation.
+b : ndarray
+    The ``B_i_j`` polynomial for pixel to focal plane transformation as ``double array[m+1][m+1]``.
     Its size must be (*m* + 1, *m* + 1) where *m* = ``B_ORDER``.
 
-ap : double array[m+1][m+1]
-    The ``AP_i_j`` polynomial for pixel to focal plane transformation.
+ap : ndarray
+    The ``AP_i_j`` polynomial for pixel to focal plane transformation as ``double array[m+1][m+1]``.
     Its size must be (*m* + 1, *m* + 1) where *m* = ``AP_ORDER``.
 
-bp : double array[m+1][m+1]
-    The ``BP_i_j`` polynomial for pixel to focal plane transformation.
+bp : ndarray
+    The ``BP_i_j`` polynomial for pixel to focal plane transformation as ``double array[m+1][m+1]``.
     Its size must be (*m* + 1, *m* + 1) where *m* = ``BP_ORDER``.
 
-crpix : double array[2]
-    The reference pixel.
+crpix : ndarray
+    The reference pixel as ``double array[2]``.
 
 Notes
 -----
@@ -1772,23 +2128,23 @@ Shupe, D. L., M. Moshir, J. Li, D. Makovoz and R. Narron.  2005.
 Headers."  ADASS XIV.
 """
 
-sip_foc2pix = """
-sip_foc2pix(*foccrd, origin*) -> double array[ncoord][nelem]
+sip_foc2pix = f"""
+sip_foc2pix(*foccrd, origin*) -> ``double array[ncoord][nelem]``
 
 Convert focal plane coordinates to pixel coordinates using the `SIP`_
 polynomial distortion convention.
 
 Parameters
 ----------
-foccrd : double array[ncoord][nelem]
-    Array of focal plane coordinates.
+foccrd : ndarray
+    Array of focal plane coordinates as ``double array[ncoord][nelem]``.
 
-{}
+{ORIGIN()}
 
 Returns
 -------
-pixcrd : double array[ncoord][nelem]
-    Returns an array of pixel coordinates.
+pixcrd : ndarray
+    Returns an array of pixel coordinates as ``double array[ncoord][nelem]``.
 
 Raises
 ------
@@ -1797,25 +2153,25 @@ MemoryError
 
 ValueError
     Invalid coordinate transformation parameters.
-""".format(ORIGIN())
+"""
 
-sip_pix2foc = """
-sip_pix2foc(*pixcrd, origin*) -> double array[ncoord][nelem]
+sip_pix2foc = f"""
+sip_pix2foc(*pixcrd, origin*) -> ``double array[ncoord][nelem]``
 
 Convert pixel coordinates to focal plane coordinates using the `SIP`_
 polynomial distortion convention.
 
 Parameters
 ----------
-pixcrd : double array[ncoord][nelem]
-    Array of pixel coordinates.
+pixcrd : ndarray
+    Array of pixel coordinates as ``double array[ncoord][nelem]``.
 
-{}
+{ORIGIN()}
 
 Returns
 -------
-foccrd : double array[ncoord][nelem]
-    Returns an array of focal plane coordinates.
+foccrd : ndarray
+    Returns an array of focal plane coordinates as ``double array[ncoord][nelem]``.
 
 Raises
 ------
@@ -1824,7 +2180,7 @@ MemoryError
 
 ValueError
     Invalid coordinate transformation parameters.
-""".format(ORIGIN())
+"""
 
 spcfix = """
 spcfix() -> int
@@ -1965,6 +2321,8 @@ axes : int or a sequence.
 
     - ``'stokes'`` / ``WCSSUB_STOKES``: Stokes axis
 
+    - ``'temporal'`` / ``WCSSUB_TIME``: Time axis (requires ``WCSLIB`` version 7.8 or greater)
+
     - ``'celestial'`` / ``WCSSUB_CELESTIAL``: An alias for the
       combination of ``'longitude'``, ``'latitude'`` and ``'cubeface'``.
 
@@ -1980,7 +2338,7 @@ MemoryError
 InvalidSubimageSpecificationError
     Invalid subimage specification (no spectral axis).
 
-NonseparableSubimageCoordinateSystem
+NonseparableSubimageCoordinateSystemError
     Non-separable subimage coordinate system.
 
 Notes
@@ -2109,7 +2467,7 @@ relax : bool or int
       standard.
 
     - `int`: a bit field selecting specific extensions to write.
-      See :ref:`relaxwrite` for details.
+      See :ref:`astropy:relaxwrite` for details.
 
 Returns
 -------
@@ -2118,9 +2476,7 @@ header : str
 """
 
 ttype = """
-``str`` (read-only)
-
-``TTYPEn`` identifying the column of the binary table that contains
+``str`` (read-only) ``TTYPEn`` identifying the column of the binary table that contains
 the wcstab array.
 """
 
@@ -2206,15 +2562,13 @@ To perform all distortion corrections and WCS transformation, use
 
 Parameters
 ----------
-sip : `~astropy.wcs.Sip` object or `None`
+sip : `~astropy.wcs.Sip` object or None
 
-cpdis : A pair of `~astropy.wcs.DistortionLookupTable` objects, or
-  ``(None, None)``.
+cpdis : (2,) tuple of `~astropy.wcs.DistortionLookupTable` or None
 
-wcsprm : `~astropy.wcs.Wcsprm` object
+wcsprm : `~astropy.wcs.Wcsprm`
 
-det2im : A pair of `~astropy.wcs.DistortionLookupTable` objects, or
-   ``(None, None)``.
+det2im : (2,) tuple of `~astropy.wcs.DistortionLookupTable` or None
 """
 
 Wcsprm = """
@@ -2236,9 +2590,22 @@ However, it does recognize free-format character (NOST 100-2.0,
 Sect. 5.2.1), integer (Sect. 5.2.3), and floating-point values
 (Sect. 5.2.4) for all keywords.
 
+
+.. warning::
+
+    Many of the attributes of this class require additional processing when
+    modifying underlying C structure.  When needed, this additional processing
+    is implemented in attribute setters. Therefore, for mutable attributes, one
+    should always set the attribute rather than a slice of its current value (or
+    its individual elements) since the latter may lead the class instance to be
+    in an invalid state.  For example, attribute ``crpix`` of a 2D WCS'
+    ``Wcsprm`` object ``wcs`` should be set as ``wcs.crpix = [crpix1, crpix2]``
+    instead of ``wcs.crpix[0] = crpix1; wcs.crpix[1] = crpix2]``.
+
+
 Parameters
 ----------
-header : An `astropy.io.fits.Header`, string, or `None`.
+header : `~astropy.io.fits.Header`, str, or None.
   If ``None``, the object will be initialized to default values.
 
 key : str, optional
@@ -2258,7 +2625,7 @@ relax : bool or int, optional
       standard.
 
     - `int`: a bit field selecting specific extensions to accept.  See
-      :ref:`relaxread` for details.
+      :ref:`astropy:relaxread` for details.
 
 naxis : int, optional
     The number of world coordinates axes for the object.  (*naxis* may
@@ -2293,6 +2660,11 @@ KeyError
      Key not found in FITS header.
 """
 
+wtb = """
+``list of Wtbarr`` objects to construct coordinate lookup tables from BINTABLE.
+
+"""
+
 Wtbarr = """
 Classes to construct coordinate lookup tables from a binary table
 extension (BINTABLE).
@@ -2366,6 +2738,12 @@ InvalidTabularParametersError()
 The given tabular parameters are invalid.
 """
 
+InvalidPrjParameters = """
+InvalidPrjParametersError()
+
+The given projection parameters are invalid.
+"""
+
 mjdbeg = """
 ``double`` Modified Julian Date corresponding to ``DATE-BEG``.
 
